package com.qmth.boot.core.retrofit.utils;

import com.qmth.boot.core.retrofit.interfaces.SignatureProvider;
import com.qmth.boot.tools.signature.SignatureType;
import org.apache.commons.lang3.StringUtils;

/**
 * 调用方动态提供的签名信息，替代容器管理的静态配置
 */
public class SignatureInfo implements SignatureProvider {

    public static final String HEADER_NAME = "signature-info";

    private static final String JOINER = "\t";

    private SignatureType type;

    private String identity;

    private String secret;

    public SignatureInfo(SignatureType type, String identity, String secret) {
        this.type = type;
        this.identity = identity;
        this.secret = secret;
    }

    @Override
    public SignatureType getType() {
        return type;
    }

    @Override
    public String getIdentity() {
        return identity;
    }

    @Override
    public String getSecret() {
        return secret;
    }

    @Override
    public String toString() {
        return type.getName() + JOINER + identity + JOINER + secret;
    }

    public static SignatureInfo parse(String value) {
        String[] values = StringUtils.split(StringUtils.trimToEmpty(value), JOINER);
        if (values.length == 3) {
            return new SignatureInfo(SignatureType.findByName(values[0]), values[1], values[2]);
        } else {
            return null;
        }
    }

}
